#include "LinuxWindow.hpp"
#include "quark/Core.hpp"
#include "quark.hpp"
#include <GLFW/glfw3.h>

namespace qk {
	static bool GLFWInitialized = false;

	std::unique_ptr<Window> Window::Create(const WindowConfig& config) {
		return std::make_unique<LinuxWindow>(config);
	}

	LinuxWindow::LinuxWindow(const WindowConfig& config) {
		Init(config);
	}

	LinuxWindow::~LinuxWindow() {
		Destroy();
	}

	void LinuxWindow::Init(const WindowConfig& config) {
		data.title = config.title;
		data.width = config.width;
		data.height = config.height;

		QK_CORE_INFO("Creating window {0} ({1}x{2})", data.title, data.width, data.height);

		if (!GLFWInitialized) {
			int success = glfwInit();
			QK_CORE_ASSERT(success, "Could not initialize GLFW");

			GLFWInitialized = true;
		}
		
		glfwWindowHintString(GLFW_X11_CLASS_NAME, "testclass");

		window = glfwCreateWindow((int)config.width, (int)config.height, config.title.c_str(), nullptr, nullptr);
		glfwMakeContextCurrent(window);
		glfwSetWindowUserPointer(window, &data);


		SetVSync(true);
	}

	void LinuxWindow::Destroy() {
		glfwDestroyWindow(window);
		window = nullptr;
	}

	void LinuxWindow::OnUpdate() {
		glfwPollEvents();
		glfwSwapBuffers(window);
	}

	constexpr void LinuxWindow::SetVSync(bool enabled) {
		if (enabled) {
			glfwSwapInterval(1);
		} else {
			glfwSwapInterval(0);
		}

		data.vsync = enabled;
	}

	bool LinuxWindow::HasVSync() const {
		return data.vsync;
	}
}
